@@ -1,6 +1,6 @@
 ////////////////////////////////////////////////////////////////////////////////
 //
-// Copyright 2016 - 2020, Thomas Lauf, Paul Beckingham, Federico Hernandez.
+// Copyright 2016 - 2023, Thomas Lauf, Paul Beckingham, Federico Hernandez.
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to deal
@@ -24,17 +24,18 @@
 //
 ////////////////////////////////////////////////////////////////////////////////
 
-#include <cmake.h>
 #include <CLI.h>
 #include <Color.h>
-#include <Pig.h>
-#include <shared.h>
-#include <format.h>
-#include <utf8.h>
-#include <sstream>
-#include <algorithm>
-#include <set>
+#include <DatetimeParser.h>
 #include <Duration.h>
+#include <Pig.h>
+#include <algorithm>
+#include <format.h>
+#include <set>
+#include <shared.h>
+#include <sstream>
+#include <timew.h>
+#include <utf8.h>
 
 ////////////////////////////////////////////////////////////////////////////////
 A2::A2 (const std::string& raw, Lexer::Type lextype)
@@ -217,7 +218,7 @@ void CLI::lexArguments ()
     if (lex.token (lexeme, type) &&
         lex.isEOS ())
     {
-      A2 a (Lexer::dequote (_original_args[i].attribute ("raw")), type);
+      A2 a (_original_args[i].attribute ("raw"), type);
       if (quoted)
         a.tag ("QUOTED");
 
@@ -521,6 +522,10 @@ void CLI::identifyFilter ()
       a.tag ("KEYWORD");
     }
 
+    else if (raw.rfind("dom.",0) == 0)
+    {
+      a.tag ("DOM");
+    }
     else
     {
       a.tag ("FILTER");
@@ -552,21 +557,21 @@ std::set<int> CLI::getIds() const
   for (auto& arg : _args)
   {
     if (arg.hasTag ("ID"))
-      ids.insert (strtol (arg.attribute ("value").c_str (), NULL, 10));
+      ids.insert (strtol (arg.attribute ("value").c_str (), nullptr, 10));
   }
 
   return ids;
 }
 
 ////////////////////////////////////////////////////////////////////////////////
-std::vector<std::string> CLI::getTags () const
+std::set <std::string> CLI::getTags () const
 {
-  std::vector <std::string> tags;
+  std::set <std::string> tags;
 
   for (auto& arg : _args)
   {
     if (arg.hasTag ("TAG"))
-      tags.push_back (arg.attribute ("raw"));
+      tags.insert (arg.attribute ("raw"));
   }
 
   return tags;
@@ -606,3 +611,245 @@ Duration CLI::getDuration () const
 }
 
 ////////////////////////////////////////////////////////////////////////////////
+
+std::vector <std::string> CLI::getDomReferences () const
+{
+  std::vector<std::string> references;
+
+  for (auto &arg : _args)
+  {
+    if (arg.hasTag ("DOM"))
+    {
+      references.emplace_back (arg.attribute ("raw"));
+    }
+  }
+
+  return references;
+}
+
+////////////////////////////////////////////////////////////////////////////////
+//
+// Supported range forms:
+//   ["from"] <date> ["to"|"-" <date>]
+//   ["from"] <date> "for" <duration>
+//   <duration> ["before"|"after" <date>]
+//   <duration> "ago"
+//
+Range CLI::getRange(const Range& default_range) const
+{
+  Datetime now;
+
+  Range the_range;
+  std::string start;
+  std::string end;
+  std::string duration;
+  std::vector <std::string> args;
+
+  for (auto& arg : _args)
+  {
+    if (arg.hasTag ("BINARY") ||
+        arg.hasTag ("CMD")    ||
+        arg.hasTag ("EXT"))
+      continue;
+
+    if (arg.hasTag ("FILTER"))
+    {
+      auto canonical = arg.attribute ("canonical");
+      auto raw       = arg.attribute ("raw");
+
+      if (arg.hasTag ("HINT"))
+      {
+        Range range;
+        if (expandIntervalHint (canonical, range))
+        {
+          if (range.is_empty ())
+          {
+            args.emplace_back("<all>");
+          }
+          else
+          {
+            start = range.start.toISO ();
+            end   = range.end.toISO ();
+
+            args.emplace_back("<date>");
+            args.emplace_back("-");
+            args.emplace_back("<date>");
+          }
+        }
+
+        // Hints that are not expandable to a date range are ignored.
+      }
+      else if (arg._lextype == Lexer::Type::date)
+      {
+        if (start.empty ())
+          start = raw;
+        else if (end.empty ())
+          end = raw;
+
+        args.emplace_back("<date>");
+      }
+      else if (arg._lextype == Lexer::Type::duration)
+      {
+        if (duration.empty ())
+          duration = raw;
+
+        args.emplace_back("<duration>");
+      }
+      else if (arg.hasTag ("KEYWORD"))
+      {
+        // Note: that KEYWORDS are not entities (why not?) and there is a list
+        //       in CLI.cpp of them that must be maintained and synced with this
+        //       function.
+        args.push_back (raw);
+      }
+    }
+  }
+
+  if (args.empty ())
+  {
+    the_range = default_range;
+  }
+  // <date>
+  else if (args.size () == 1 &&
+           args[0] == "<date>")
+  {
+    DatetimeParser dtp;
+    Range range = dtp.parse_range(start);
+    the_range = Range (range);
+  }
+  // from <date>
+  else if (args.size () == 2 &&
+           args[0] == "from" &&
+           args[1] == "<date>")
+  {
+    the_range = Range ({Datetime (start), 0});
+  }
+  // <date> to/- <date>
+  else if (args.size () == 3                   &&
+           args[0] == "<date>"                 &&
+           (args[1] == "to" || args[1] == "-") &&
+           args[2] == "<date>")
+  {
+    the_range = Range ({Datetime (start), Datetime (end)});
+  }
+  // from <date> to/- <date>
+  else if (args.size () == 4                   &&
+           args[0] == "from"                   &&
+           args[1] == "<date>"                 &&
+           (args[2] == "to" || args[2] == "-") &&
+           args[3] == "<date>")
+  {
+    the_range = Range ({Datetime (start), Datetime (end)});
+  }
+  // <date> for <duration>
+  else if (args.size () == 3   &&
+           args[0] == "<date>" &&
+           args[1] == "for"    &&
+           args[2] == "<duration>")
+  {
+    the_range = Range ({Datetime (start), Datetime (start) + Duration (duration).toTime_t ()});
+  }
+  // from <date> for <duration>
+  else if (args.size () == 4       &&
+           args[0] == "from"       &&
+           args[1] == "<date>"     &&
+           args[2] == "for"        &&
+           args[3] == "<duration>")
+  {
+    the_range = Range ({Datetime (start), Datetime (start) + Duration (duration).toTime_t ()});
+  }
+  // <duration> before <date>
+  else if (args.size () == 3       &&
+           args[0] == "<duration>" &&
+           args[1] == "before"     &&
+           args[2] == "<date>")
+  {
+    the_range = Range ({Datetime (start) - Duration (duration).toTime_t (), Datetime (start)});
+  }
+  // <duration> after <date>
+  else if (args.size () == 3       &&
+           args[0] == "<duration>" &&
+           args[1] == "after"      &&
+           args[2] == "<date>")
+  {
+    the_range = Range ({Datetime (start), Datetime (start) + Duration (duration).toTime_t ()});
+  }
+  // <duration> ago
+  else if (args.size () == 2       &&
+           args[0] == "<duration>" &&
+           args[1] == "ago")
+  {
+    the_range = Range ({now - Duration (duration).toTime_t (), 0});
+  }
+  // for <duration>
+  else if (args.size () == 2       &&
+           args[0] == "for"        &&
+           args[1] == "<duration>")
+  {
+    the_range = Range ({now - Duration (duration).toTime_t (), now});
+  }
+  // <duration>
+  else if (args.size () == 1 &&
+           args[0] == "<duration>")
+  {
+    the_range = Range ({now - Duration (duration).toTime_t (), now});
+  }
+  // :all
+  else if (args.size () == 1 && args[0] == "<all>")
+  {
+    the_range = Range (0, 0);
+  }
+  // Unrecognized date range construct.
+  else if (! args.empty ())
+  {
+    throw std::string ("Unrecognized date range: '") + join (" ", args) + "'.";
+  }
+
+  if (the_range.end != 0 && the_range.start > the_range.end)
+  {
+    throw std::string ("The end of a date range must be after the start.");
+  }
+
+  return the_range;
+}
+
+////////////////////////////////////////////////////////////////////////////////
+bool CLI::findHint (const std::string& hint) const
+{
+  for (auto& arg : _args)
+  {
+    if (arg.hasTag ("HINT") &&
+        arg.getToken () == ":" + hint)
+    {
+      return true;
+    }
+  }
+
+  return false;
+}
+
+////////////////////////////////////////////////////////////////////////////////
+bool CLI::getComplementaryHint (const std::string& base, const bool default_value) const
+{
+  if (findHint (base))
+  {
+    return true;
+  }
+  else if (findHint ("no-" + base))
+  {
+    return false;
+  }
+
+  return default_value;
+}
+
+////////////////////////////////////////////////////////////////////////////////
+bool CLI::getHint (const std::string &base, const bool default_value) const
+{
+  if (findHint (base))
+  {
+    return true;
+  }
+
+  return default_value;
+}
