@@ -1,6 +1,6 @@
 ////////////////////////////////////////////////////////////////////////////////
 //
-// Copyright 2015 - 2020, Thomas Lauf, Paul Beckingham, Federico Hernandez.
+// Copyright 2015 - 2023, Thomas Lauf, Paul Beckingham, Federico Hernandez.
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to deal
@@ -24,18 +24,17 @@
 //
 ////////////////////////////////////////////////////////////////////////////////
 
-#include <cmake.h>
-#include <Rules.h>
+#include <AtomicFile.h>
 #include <FS.h>
-#include <shared.h>
-#include <format.h>
-#include <sstream>
-#include <tuple>
+#include <JSON.h>
+#include <Rules.h>
 #include <cassert>
 #include <cerrno>
-#include <inttypes.h>
-#include <AtomicFile.h>
-#include <JSON.h>
+#include <cinttypes>
+#include <format.h>
+#include <shared.h>
+#include <sstream>
+#include <tuple>
 
 ////////////////////////////////////////////////////////////////////////////////
 Rules::Rules ()
@@ -90,15 +89,15 @@ Rules::Rules ()
     // 'summary' report.
     {"reports.summary.holidays", "yes"},
 
-    // 'gaps' report.
-    {"reports.gaps.range",       ":day"},
-
     // Enough of a theme to make the charts work.
     {"theme.description",        "Built-in default"},
     {"theme.colors.exclusion",   "gray8 on gray4"},
     {"theme.colors.today",       "white"},
     {"theme.colors.holiday",     "gray4"},
     {"theme.colors.label",       "gray4"},
+
+    // Options for the journal / undo file.
+    {"journal.size",             "-1"},
   };
 }
 
@@ -195,9 +194,10 @@ double Rules::getReal (const std::string& key) const
 }
 
 ////////////////////////////////////////////////////////////////////////////////
-bool Rules::getBoolean (const std::string& key) const
+bool Rules::getBoolean (const std::string& key, bool defaultValue) const
 {
   auto found = _settings.find (key);
+
   if (found != _settings.end ())
   {
     auto value = lowerCase (found->second);
@@ -206,10 +206,14 @@ bool Rules::getBoolean (const std::string& key) const
         value == "y"      ||
         value == "yes"    ||
         value == "on")
+    {
       return true;
+    }
+
+    return false;
   }
 
-  return false;
+  return defaultValue;
 }
 
 ////////////////////////////////////////////////////////////////////////////////
@@ -463,7 +467,7 @@ void Rules::parseRuleSettings (
 ////////////////////////////////////////////////////////////////////////////////
 unsigned int Rules::getIndentation (const std::string& line)
 {
-  auto indent = line.find_first_not_of (" ");
+  auto indent = line.find_first_not_of (' ');
   if (indent == std::string::npos)
     indent = 0;
 
@@ -601,7 +605,7 @@ bool Rules::setConfigVariable (
       {
         // Add blank line required by rules.
         if (lines.empty () || lines.back ().empty ())
-          lines.push_back ("");
+          lines.emplace_back ("");
 
         // Add new line.
         lines.push_back (name + " = " + json::encode (value));
@@ -622,7 +626,7 @@ bool Rules::setConfigVariable (
 
       // Add blank line required by rules.
       if (lines.empty () || lines.back ().empty ())
-        lines.push_back ("");
+        lines.emplace_back ("");
 
       // Add new line.
       lines.push_back (name + " = " + json::encode (value));
