@@ -1,6 +1,6 @@
 ////////////////////////////////////////////////////////////////////////////////
 //
-// Copyright 2016 - 2020, Thomas Lauf, Paul Beckingham, Federico Hernandez.
+// Copyright 2016 - 2023, Thomas Lauf, Paul Beckingham, Federico Hernandez.
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to deal
@@ -24,204 +24,14 @@
 //
 ////////////////////////////////////////////////////////////////////////////////
 
-#include <algorithm>
-#include <deque>
-
-#include <cmake.h>
-#include <shared.h>
-#include <format.h>
 #include <Datetime.h>
 #include <Duration.h>
-#include <timew.h>
-#include <algorithm>
-#include <iostream>
 #include <IntervalFactory.h>
-
-////////////////////////////////////////////////////////////////////////////////
-// A filter is just another interval, containing start, end and tags.
-//
-// Supported interval forms:
-//   ["from"] <date> ["to"|"-" <date>]
-//   ["from"] <date> "for" <duration>
-//   <duration> ["before"|"after" <date>]
-//   <duration> "ago"
-//
-Interval getFilter (const CLI& cli)
-{
-  // One instance, so we can directly compare.
-  Datetime now;
-
-  Interval filter;
-  std::string start;
-  std::string end;
-  std::string duration;
-  std::vector <std::string> args;
-  for (auto& arg : cli._args)
-  {
-    if (arg.hasTag ("BINARY") ||
-        arg.hasTag ("CMD")    ||
-        arg.hasTag ("EXT"))
-      continue;
-
-    if (arg.hasTag ("FILTER"))
-    {
-      auto canonical = arg.attribute ("canonical");
-      auto raw       = arg.attribute ("raw");
-
-      if (arg.hasTag ("HINT"))
-      {
-        Range range;
-        if (expandIntervalHint (canonical, range))
-        {
-          start = range.start.toISO ();
-          end   = range.end.toISO ();
-
-          args.push_back ("<date>");
-          args.push_back ("-");
-          args.push_back ("<date>");
-        }
-
-        // Hints that are not expandable to a date range are ignored.
-      }
-      else if (arg._lextype == Lexer::Type::date)
-      {
-        if (start.empty ())
-          start = raw;
-        else if (end.empty ())
-          end = raw;
-
-        args.push_back ("<date>");
-      }
-      else if (arg._lextype == Lexer::Type::duration)
-      {
-        if (duration.empty ())
-          duration = raw;
-
-        args.push_back ("<duration>");
-      }
-      else if (arg.hasTag ("KEYWORD"))
-      {
-        // Note: that KEYWORDS are not entities (why not?) and there is a list
-        //       in CLI.cpp of them that must be maintained and synced with this
-        //       function.
-        args.push_back (raw);
-      }
-      else if (arg.hasTag ("ID"))
-      {
-        // Not part of a filter.
-      }
-      else
-      {
-        filter.tag (raw);
-      }
-    }
-  }
-
-  // <date>
-  if (args.size () == 1 &&
-      args[0] == "<date>")
-  {
-    filter.setRange ({Datetime (start), 0});
-  }
-
-  // from <date>
-  else if (args.size () == 2 &&
-           args[0] == "from" &&
-           args[1] == "<date>")
-  {
-    filter.setRange ({Datetime (start), 0});
-  }
-
-  // <date> to/- <date>
-  else if (args.size () == 3                   &&
-           args[0] == "<date>"                 &&
-           (args[1] == "to" || args[1] == "-") &&
-           args[2] == "<date>")
-  {
-    filter.setRange ({Datetime (start), Datetime (end)});
-  }
-
-  // from <date> to/- <date>
-  else if (args.size () == 4                   &&
-           args[0] == "from"                   &&
-           args[1] == "<date>"                 &&
-           (args[2] == "to" || args[2] == "-") &&
-           args[3] == "<date>")
-  {
-    filter.setRange ({Datetime (start), Datetime (end)});
-  }
-
-  // <date> for <duration>
-  else if (args.size () == 3   &&
-           args[0] == "<date>" &&
-           args[1] == "for"    &&
-           args[2] == "<duration>")
-  {
-    filter.setRange ({Datetime (start), Datetime (start) + Duration (duration).toTime_t ()});
-  }
-
-  // from <date> for <duration>
-  else if (args.size () == 4       &&
-           args[0] == "from"       &&
-           args[1] == "<date>"     &&
-           args[2] == "for"        &&
-           args[3] == "<duration>")
-  {
-    filter.setRange ({Datetime (start), Datetime (start) + Duration (duration).toTime_t ()});
-  }
-
-  // <duration> before <date>
-  else if (args.size () == 3       &&
-           args[0] == "<duration>" &&
-           args[1] == "before"     &&
-           args[2] == "<date>")
-  {
-    filter.setRange ({Datetime (start) - Duration (duration).toTime_t (), Datetime (start)});
-  }
-
-  // <duration> after <date>
-  else if (args.size () == 3       &&
-           args[0] == "<duration>" &&
-           args[1] == "after"      &&
-           args[2] == "<date>")
-  {
-    filter.setRange ({Datetime (start), Datetime (start) + Duration (duration).toTime_t ()});
-  }
-
-  // <duration> ago
-  else if (args.size () == 2       &&
-           args[0] == "<duration>" &&
-           args[1] == "ago")
-  {
-    filter.setRange ({now - Duration (duration).toTime_t (), 0});
-  }
-
-  // for <duration>
-  else if (args.size () == 2       &&
-           args[0] == "for"        &&
-           args[1] == "<duration>")
-  {
-    filter.setRange ({now - Duration (duration).toTime_t (), now});
-  }
-
-  // <duration>
-  else if (args.size () == 1 &&
-           args[0] == "<duration>")
-  {
-    filter.setRange ({now - Duration (duration).toTime_t (), now});
-  }
-
-  // Unrecognized date range construct.
-  else if (! args.empty ())
-  {
-    throw std::string ("Unrecognized date range: '") + join (" ", args) + "'.";
-  }
-
-  if (filter.end != 0 && filter.start > filter.end)
-    throw std::string ("The end of a date range must be after the start.");
-
-  return filter;
-}
+#include <IntervalFilter.h>
+#include <algorithm>
+#include <format.h>
+#include <shared.h>
+#include <timew.h>
 
 ////////////////////////////////////////////////////////////////////////////////
 // Read rules and extract all holiday definitions. Create a Range for each
@@ -270,7 +80,7 @@ std::vector <Range> getAllExclusions (
   // Load all exclusions from configuration.
   std::vector <Exclusion> exclusions;
   for (auto& name : rules.all ("exclusions."))
-    exclusions.push_back (Exclusion (lowerCase (name), rules.get (name)));
+    exclusions.emplace_back (lowerCase (name), rules.get (name));
   debug (format ("Found {1} exclusions", exclusions.size ()));
 
   // Find exclusions 'exc day on <date>' and remove from holidays.
@@ -304,151 +114,78 @@ std::vector <Range> getAllExclusions (
   // overlap with range.
   std::vector <Range> exclusionRanges;
   for (auto& exclusion : exclusions)
+  {
     if (exclusion.tokens ()[1] != "day")
+    {
       for (auto& r : exclusion.ranges (range))
-        exclusionRanges.push_back (r);
+      {
+        if (!r.is_empty ())
+        {
+          exclusionRanges.push_back (r);
+        }
+      }
+    }
+  }
 
   return merge (addRanges (range, results, exclusionRanges));
 }
 
 ////////////////////////////////////////////////////////////////////////////////
-void flattenDatabase (Database& database, const Rules& rules)
+// Potentially expand the latest interval into a collection of synthetic
+// intervals.
+std::vector <Interval> expandLatest (const Interval& latest, const Rules& rules)
 {
-  auto latest = getLatestInterval (database);
-  if (latest.is_open ())
-  {
-    auto exclusions = getAllExclusions (rules, {latest.start, Datetime ()});
-    if (! exclusions.empty ())
-    {
-      Interval modified {latest};
-
-      // Update database.
-      database.deleteInterval (latest);
-      for (auto& interval : flatten (modified, exclusions))
-      {
-        database.addInterval (interval, rules.getBoolean ("verbose"));
-      }
-    }
-  }
-}
-
-////////////////////////////////////////////////////////////////////////////////
-// This function will return synthetic intervals
-std::vector <Interval> getIntervalsByIds (
-  Database& database,
-  const Rules& rules,
-  const std::set <int>& ids)
-{
-  std::vector <Interval> intervals;
-  std::deque <Interval> synthetic;
-
   int current_id = 0;
-  auto id_it = ids.begin ();
-  auto id_end = ids.end ();
-
-  auto it = database.begin ();
-  auto end = database.end ();
-
-  Interval latest;
-  if (it != end )
-  {
-    latest = IntervalFactory::fromSerialization (*it);
-  }
-
+  std::vector <Interval> intervals;
   // If the latest interval is open, check for synthetic intervals
   if (latest.is_open ())
   {
     auto exclusions = getAllExclusions (rules, {latest.start, Datetime ()});
     if (! exclusions.empty ())
     {
-        // We're converting the latest interval into synthetic intervals so we need to skip it
-        ++it;
+      std::vector <Interval> flattened = flatten (latest, exclusions);
 
-        for (auto& interval : flatten (latest, exclusions))
+      // If flatten() converted the latest interval into a group of synthetic
+      // intervals, the number of returned intervals will be greater than 1.
+      // Otherwise, it just returned the non-synthetic, latest interval.
+      if (flattened.size () > 1)
+      {
+        std::reverse (flattened.begin (), flattened.end ());
+        for (auto interval : flattened)
         {
-          ++current_id;
           interval.synthetic = true;
-          interval.id = current_id;
-          synthetic.push_front (interval);
+          interval.id = ++current_id;
+          intervals.push_back (std::move (interval));
         }
+      }
     }
   }
 
-  intervals.reserve (ids.size ());
-
-  // First look for the ids in our set of synthetic intervals
-  for (auto& interval : synthetic)
+  if (intervals.empty ())
   {
-    if (id_it == id_end)
-    {
-      break;
-    }
-
-    if (interval.id == *id_it)
-    {
-      intervals.push_back (interval);
-      ++id_it;
-    }
+    intervals.push_back (latest);
+    intervals.back().id = 1;
   }
-
-  current_id = synthetic.size ();
-
-  // We'll find remaining intervals from the database itself
-  for ( ; it != end; ++it)
-  {
-    ++current_id;
-
-    if (id_it == id_end)
-    {
-      break;
-    }
-
-    if (current_id == *id_it)
-    {
-      Interval interval = IntervalFactory::fromSerialization (*it);
-      interval.id = current_id;
-      intervals.push_back (interval);
-      ++id_it;
-    }
-  }
-
-  // We did not find all the ids we were looking for.
-  if (id_it != id_end)
-  {
-    throw format("ID '@{1}' does not correspond to any tracking.", *id_it);
-  }
-
   return intervals;
 }
 
 ////////////////////////////////////////////////////////////////////////////////
-std::vector <Interval> subset (
-  const Interval& filter,
-  const std::deque <Interval>& intervals)
+// Convert what would be synthetic intervals into real intervals in the database
+void flattenDatabase (Database& database, const Rules& rules)
 {
-  std::vector <Interval> all;
-  for (auto& interval : intervals)
+  Interval latest = getLatestInterval (database);
+  std::vector <Interval> expanded = expandLatest (latest, rules);
+
+  if (expanded.size () > 1)
   {
-    if (matchesFilter (interval, filter))
+    bool verbose = rules.getBoolean ("verbose");
+    database.deleteInterval (latest);
+    for (auto it = expanded.rbegin (); it != expanded.rend (); ++it)
     {
-      all.push_back (interval);
+      it->synthetic = false;
+      database.addInterval (*it, verbose);
     }
   }
-
-  return all;
-}
-
-////////////////////////////////////////////////////////////////////////////////
-std::vector <Interval> subset (
-  const Interval& filter,
-  const std::vector <Interval>& intervals)
-{
-  std::vector <Interval> all;
-  for (auto& interval : intervals)
-    if (matchesFilter (interval, filter))
-      all.push_back (interval);
-
-  return all;
 }
 
 ////////////////////////////////////////////////////////////////////////////////
@@ -497,6 +234,11 @@ std::vector <Interval> flatten (
   Datetime now;
   for (auto& result : subtractRanges ({interval}, enclosed))
   {
+    if (interval.is_open() && result.start > now)
+    {
+      break;
+    }
+
     Interval chunk {interval};
     chunk.setRange (result);
 
@@ -516,7 +258,7 @@ static bool rangeCompare (Range left, Range right)
 std::vector <Range> merge (
   const std::vector <Range>& ranges)
 {
-  // Short cut.
+  // Shortcut.
   if (ranges.size () < 2)
     return ranges;
 
@@ -589,9 +331,7 @@ std::vector <Range> subtractRanges (
 // An interval matches a filter range if the start/end overlaps
 bool matchesRange (const Interval& interval, const Range& filter)
 {
-  return ((filter.start.toEpoch () == 0 &&
-           filter.end.toEpoch () == 0
-          ) || interval.intersects (filter));
+  return ((!filter.is_started() && !filter.is_ended()) || interval.intersects (filter));
 }
 
 ////////////////////////////////////////////////////////////////////////////////
@@ -669,112 +409,66 @@ Interval clip (const Interval& interval, const Range& range)
 }
 
 ////////////////////////////////////////////////////////////////////////////////
+// Return collection of intervals that match the filter (synthetic intervals
+// included) sorted by date
 std::vector <Interval> getTracked (
   Database& database,
   const Rules& rules,
-  Interval& filter)
+  IntervalFilter& filter)
 {
-  // Exclusions are only usable within a range, so if no filter range exists,
-  // determine the infinite range starting at the first inclusion, i.e.:
-  //
-  //   [earliest start, infinity)
-  //
-  // Avoid assigning a zero-width range - leave it unstarted instead.
-  if (! filter.is_started ())
+  int current_id = 0;
+  std::vector <Interval> intervals;
+
+  auto it = database.begin ();
+  auto end = database.end ();
+
+  // Because the latest recorded interval may be expanded into synthetic
+  // intervals, we'll handle it specially
+  if (it != end )
   {
-    auto begin = database.rbegin ();
-    auto end = database.rend ();
-    if (begin != end)
+    Interval latest = IntervalFactory::fromSerialization (*it);
+    ++it;
+
+    for (auto& interval : expandLatest (latest, rules))
     {
-      filter.start = IntervalFactory::fromSerialization (*begin).start;
-    }
-
-    // Use an infinite range instead of the last end date; this prevents
-    // issues when there is an empty range [q, q) at the end of a filter
-    // [p, q), in which case there is no overlap or intersection.
-    filter.end = 0;
-  }
-
-  int id_skip = 0;
-  std::deque <Interval> intervals;
-
-  for (auto& line : database)
-  {
-    Interval interval = IntervalFactory::fromSerialization(line);
-
-    // Since we are moving backwards, and the intervals are in sorted order,
-    // if the filter is after the interval, we know there will be no more matches
-    if (matchesRange (interval, filter))
-    {
-      intervals.push_front (std::move (interval));
-    }
-    else if (interval.start.toEpoch () >= filter.start.toEpoch ())
-    {
-      ++id_skip;
-    }
-    else
-    {
-      break;
-    }
-  }
-
-  if (! intervals.empty ())
-  {
-    auto latest = intervals.back ();
-
-    if (latest.is_open ())
-    {
-      // Get the set of expanded exclusions that overlap the range defined by the open interval.
-      Interval exclusion_range {};
-      exclusion_range.start = latest.start;
-      exclusion_range.end = Datetime();
-
-      auto exclusions = getAllExclusions (rules, exclusion_range);
-      if (! exclusions.empty ())
+      ++current_id;
+      if (filter.accepts (interval))
       {
-        intervals.pop_back ();
-
-        for (auto& interval : flatten (latest, exclusions))
-        {
-          if (latest.synthetic ||
-              latest != interval)
-            interval.synthetic = true;
-
-          intervals.push_back (interval);
-        }
+        interval.id = current_id;
+        intervals.push_back (interval);
+      }
+      else if (filter.is_done ())
+      {
+        break;
       }
     }
   }
 
-  // Assign an ID to each interval before we prune ones that do not have matching tags
-  for (unsigned int i = 0; i < intervals.size (); ++i)
+  for (; it != end; ++it)
   {
-    intervals[i].id = intervals.size () - i + id_skip;
-  }
+    Interval interval = IntervalFactory::fromSerialization(*it);
+    interval.id = ++current_id;
 
-  // Now prune the intervals without matching tags
-  if (filter.tags ().size () > 0)
-  {
-    auto cmp = [&filter] (const Interval& interval)
-               {
-                 for (const auto& tag : filter.tags ())
-                 {
-                   if (! interval.hasTag (tag))
-                   {
-                     return true;
-                   }
-                 }
-                 return false;
-               };
-
-    intervals.erase (std::remove_if (intervals.begin (), intervals.end (), cmp),
-                     intervals.end ());
+    if (filter.accepts (interval))
+    {
+      intervals.push_back (std::move (interval));
+    }
+    else if (filter.is_done ())
+    {
+      // Since we are moving backwards in time, and the intervals are in sorted
+      // order, if the filter is after the interval, we know there will be no
+      // more matches
+      break;
+    }
   }
 
   debug (format ("Loaded {1} tracked intervals", intervals.size ()));
 
-  return std::vector <Interval> (std::make_move_iterator (intervals.begin ()),
-                                 std::make_move_iterator (intervals.end ()));
+  // By default, intervals are sorted by id, but getTracked needs to return the
+  // intervals sorted by date, which are ids in reverse order.
+  std::reverse (intervals.begin (), intervals.end ());
+
+  return intervals;
 }
 
 ////////////////////////////////////////////////////////////////////////////////
@@ -797,7 +491,7 @@ std::vector <Range> getUntracked (
     else if (found_match)
     {
       // If we already had a match, and now we do not, since the database is in
-      // order from most recent to oldest inclusion, we can be sure that there
+      // order from most recent to the oldest inclusion, we can be sure that there
       // will not be any further matches.
       break;
     }
